//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Play_Library, Test Game
//
// Date: 13th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
// This application shows how to create a connection, create a session,
// create players, send messages between players and finally, how to cleanup
// successfully.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#include <dos.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
#include "TPlayerSetupForm.h"
#include "TGameForm.h"
#include "TStatusForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Play"
#pragma link "TDx_Play_Library_Install"
#pragma link "TDx_PlayLobby"
#pragma link "TDPName"
#pragma link "TDPSessionDesc"
#pragma link "TDPCaps"
#pragma link "TDPChat"
#pragma link "TDPApplicationDesc"
#pragma link "TDPLConnection"
#pragma link "TDPLAppInfo"
#pragma link "TDPCompoundAddressElement"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
// GUID for sessions this application creates
DEFINE_GUID( MY_SESSION_GUID, 0xd559fc00, 0xdc12, 0x11cf, 0x9c, 0x4e, 0x0, 0xa0, 0xc9, 0x5, 0x42, 0x5e );

//---------------------------------------------------------------------------
AnsiString HexStr( unsigned int hexvalue )
{
char* hexchars[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};

AnsiString string = "0x";
int value = 0x10000000;

for (int i=0;i<8;i++)
    {
    string += hexchars[hexvalue/value];
    hexvalue -= value * (hexvalue/value);
    value = value >> 4;
    }
return string;
}

AnsiString GUIDToStr( GUID guid )
{
//typedef struct  _GUID
//    {
//    DWORD Data1;
//    WORD Data2;
//    WORD Data3;
//    BYTE Data4[ 8 ];
//    }	GUID

AnsiString data1 = HexStr( guid.Data1 );
AnsiString data2 = HexStr( guid.Data2 );
AnsiString data3 = HexStr( guid.Data3 );
AnsiString data4 = "";
for (int i=0;i<8;i++)
	data4 = data4 + HexStr( guid.Data4[i] );
return data1 + "::" + data2 + "::" + data3 + "::" + data4;
}

//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
NumConnections=0;
NumSessions=0;
NumApplications=0;
NumAddresses=0;
NumPlayers = 0;
NumGroups = 0;
SessionOpen=false;
I_Am_The_Host=false;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
while (NumPlayers>0)
	{
    if (PlayerNames[NumPlayers-1]) delete PlayerNames[NumPlayers-1];
    NumPlayers--;
    }
while (NumGroupsInGroups>0)
	{
    if (GroupsInGroupsNames[NumGroupsInGroups-1]) delete GroupsInGroupsNames[NumGroupsInGroups-1];
    NumGroupsInGroups--;
    }
while (NumGroups>0)
	{
    if (GroupNames[NumGroups-1]) delete GroupNames[NumGroups-1];
    NumGroups--;
    }
while (NumSessions>0)
	{
    if (Sessions[NumSessions-1]) delete Sessions[NumSessions-1];
    NumSessions--;
    }
while (NumConnections>0)
	{
	if (ConnectionNames[NumConnections-1]) delete ConnectionNames[NumConnections-1];
	if (ConnectionBuffers[NumConnections-1]) delete ConnectionBuffers[NumConnections-1];
    NumConnections--;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Dx_Play1Error(TObject *Sender, AnsiString Function,
	AnsiString Error, AnsiString ErrorMessage)
{
if (Dx_Play1->ErrorValue!=DPERR_NOMESSAGES)
	{
    StatusForm->ErrorListBox->Items->Add( "-----------------" );
	StatusForm->ErrorListBox->Items->Add( "Error: "+Error );
	StatusForm->ErrorListBox->Items->Add( "Code: "+IntToStr((dword)Dx_Play1->ErrorValue) );
	StatusForm->ErrorListBox->Items->Add( "Function: "+Function );
	StatusForm->ErrorListBox->Items->Add( "Message: "+ErrorMessage );
    StatusForm->ErrorListBox->ItemIndex = StatusForm->ErrorListBox->Items->Count-1;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumConnections(TObject *Sender,
	const GUID *SP, void *Connection, DWORD ConnectionSize, TDPName *Name,
	DWORD Flags, void *Context, bool &Continue)
{
if (NumConnections<MAX_CONNECTIONS)
    {
    // save information for later use
    ConnectionGuids[NumConnections] = *SP;
    ConnectionBuffers[NumConnections] = new byte[ConnectionSize];
    if (ConnectionBuffers[NumConnections])
       CopyMemory( ConnectionBuffers[NumConnections], Connection, ConnectionSize );
    ConnectionSizes[NumConnections] = ConnectionSize;
    ConnectionNames[NumConnections] = new TDPName(this);
    if (ConnectionNames[NumConnections])
    	{
    	ConnectionNames[NumConnections]->LongNameA = Name->LongNameA;
    	ConnectionNames[NumConnections]->ShortNameA = Name->ShortNameA;
        }
    ConnectionFlags[NumConnections] = Flags;

    // display stuff
	ConnectionsListBox->Items->Add( "[Connection #"+IntToStr(NumConnections)+"] "+Name->ShortNameA );
	NumConnections++;
	}
else
	Continue = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumSessions(TObject *Sender,
	TDPSessionDesc *ThisSD, DWORD *TimeOut, DWORD Flags, void *Context,
	bool &Continue)
{
if (NumSessions<MAX_SESSIONS)
	{
    SessionsListBox->Items->Add( ThisSD->SessionNameA );

	Sessions[NumSessions] = new TDPSessionDesc(this);
    CopyMemory( Sessions[NumSessions]->Internal_DPSESSIONDESC2_Ptr, ThisSD->Internal_DPSESSIONDESC2_Ptr, sizeof(DPSESSIONDESC2) );

    NumSessions++;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BitBtn8Click(TObject *Sender)
{
if (Dx_Play1->Created) Dx_Play1->Destroy();
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Receive(TObject *Sender)
{
StatusForm->EventListBox->Items->Add("OnReceive");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateSessionButtonClick(TObject *Sender)
{
DPSessionDesc1->SessionNameA = SessionNameEdit->Text;
DPSessionDesc1->MaxPlayers = StrToInt( MaxPlayersEdit->Text );
DPSessionDesc1->Flags = DPSESSION_KEEPALIVE | DPSESSION_MIGRATEHOST | DPSESSION_DIRECTPLAYPROTOCOL;
if (Dx_Play1->Open( DPSessionDesc1, DPOPEN_CREATE ))
    {
    SessionOpen = true;
    I_Am_The_Host = true;
    StatusForm->SessionStatusLabel->Caption = DPSessionDesc1->SessionNameA;
    StatusForm->MaxPlayersLabel->Caption = IntToStr((int)DPSessionDesc1->MaxPlayers) + " Players";
    ShowMessage("Session Created");
    Visible = false;
    PlayerSetupForm->ShowModal();
    Visible = true;
    CloseSessionButtonClick(this);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1StartSession(TObject *Sender,
   TDPLConnection *Conn)
{
StatusForm->EventListBox->Items->Add("OnStartSession");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1SessionLost(TObject *Sender)
{
StatusForm->EventListBox->Items->Add("OnSessionLost");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Create(TObject *Sender)
{
StatusForm->ErrorListBox->Items->Clear();

// display status
StatusForm->PlayStatusLabel->Caption = "Dx_Play1 is created";

// Enumerate available connections
ConnectionsListBox->Items->Clear();
NumConnections = 0;
Dx_Play1->EnumConnections( NULL, NULL, 0 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Destroy(TObject *Sender)
{
if (SessionOpen)
	{
    ShowMessage("Closing Session");
    Dx_Play1->Close();
    }

// update status
StatusForm->PlayStatusLabel->Caption = "Dx_Play1 not created";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Generic(TObject *Sender, DWORD From,
   DWORD To, DWORD Flags, void *Data, DWORD DataSize, DWORD MessageType)
{
StatusForm->EventListBox->Items->Clear();
StatusForm->EventListBox->Items->Add("OnGeneric");

if (I_Am_The_Host)
	{
    if (To==MainForm->PlayerIDs[0])
    	{
	    GameForm->MessageListBox->Items->Clear();
        GameForm->MessageListBox->Items->Add("The Game Host Has Received a Message");
        }
    }

MessageStruct* message = (MessageStruct*) Data;
GameInfoStruct* game_info = (GameInfoStruct*) &message->Message_Data;
if (message)
	{
    switch (message->Message_Type)
    	{
		case GAME_TICK_MESSAGE:		// 0
        	{
        	StatusForm->EventListBox->Items->Add( "Got a game update tick" );
//			StatusForm->EventListBox->Items->Add("From: "+IntToStr((int)From));
//			StatusForm->EventListBox->Items->Add("To: "+IntToStr((int)To));
//			StatusForm->EventListBox->Items->Add("Flags: "+IntToStr((int)Flags));
//			StatusForm->EventListBox->Items->Add("Data: "+IntToStr((int)Data));

//                StatusForm->EventListBox->Items->Add( "  x: "+IntToStr(game_info->x) );
//                StatusForm->EventListBox->Items->Add( "  y: "+IntToStr(game_info->y) );
//                StatusForm->EventListBox->Items->Add( "  color: "+IntToStr((int)game_info->color) );
//                StatusForm->EventListBox->Items->Add( "  score: "+IntToStr(game_info->score) );
//                StatusForm->EventListBox->Items->Add( (game_info->Is_Hit) ? "  is_hit: true" : "  is_hit: false" );
//                StatusForm->EventListBox->Items->Add( (game_info->Is_Flashing) ? "  is_flashing: true" : "  is_flashing: false" );
//                StatusForm->EventListBox->Items->Add( (game_info->Is_Running) ? "  is_running: true" : "  is_running: false" );
//                StatusForm->EventListBox->Items->Add( "  Interval: "+IntToStr(game_info->Interval) );

                if (GameForm)
                	{
                    CopyMemory( &GameForm->GameInfo, game_info, message->Message_Length );
                    GameForm->Timer1Timer(this);
                    }

//			StatusForm->EventListBox->Items->Add("DataSize: "+IntToStr((int)DataSize));
//			StatusForm->EventListBox->Items->Add("MessageType: "+IntToStr((int)MessageType));
            }
        	break;

		case PLAYER_CLICK_MESSAGE:	// 1
        	{
        	StatusForm->EventListBox->Items->Add( "Got a game player click message" );
            if (GameForm)
            	{
                CopyMemory( &GameForm->GameInfo, game_info, message->Message_Length );
                GameForm->Timer1Timer(this);
                }
            }
        	break;

		case PLAYER_TEXT_MESSAGE:	// 2
        	{
			StatusForm->EventListBox->Items->Add("From: "+IntToStr((int)From));
			StatusForm->EventListBox->Items->Add("To: "+IntToStr((int)To));
			StatusForm->EventListBox->Items->Add("Flags: "+IntToStr((int)Flags));
			StatusForm->EventListBox->Items->Add("Data: "+IntToStr((int)Data));
			char* ch = (char*) &message->Message_Data;
            AnsiString temp = ch;
			StatusForm->EventListBox->Items->Add("Data[AsString]: "+temp);
			StatusForm->EventListBox->Items->Add("DataSize: "+IntToStr((int)DataSize));
			StatusForm->EventListBox->Items->Add("MessageType: "+IntToStr((int)MessageType));
            }
        	break;
        }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::JoinSessionButtonClick(TObject *Sender)
{
if (Dx_Play1->Open( Sessions[SessionsListBox->ItemIndex], DPOPEN_JOIN ))
   	{
	SessionOpen = true;
    I_Am_The_Host = false;
	StatusForm->SessionStatusLabel->Caption = Sessions[SessionsListBox->ItemIndex]->SessionNameA;
    ShowMessage("Session Joined");
    Visible = false;
    PlayerSetupForm->ShowModal();
    Visible = true;
    CloseSessionButtonClick(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CloseSessionButtonClick(TObject *Sender)
{
if (Dx_Play1->Close())
    {
    SessionOpen = false;
	StatusForm->SessionStatusLabel->Caption = "(no session)";
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1AddGroupToGroup(TObject *Sender,
	DWORD IdParentGroup, DWORD IdGroup)
{
StatusForm->EventListBox->Items->Add("OnAddGroupToGroup");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1AddPlayerToGroup(TObject *Sender,
	DWORD IdParentGroup, DWORD IdGroup)
{
StatusForm->EventListBox->Items->Add("OnAddPlayerToGroup");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Chat(TObject *Sender, DWORD Flags,
	DWORD FromPlayer, DWORD ToPlayer, DWORD ToGroup, TDPChat *Chat)
{
StatusForm->EventListBox->Items->Add("OnChat");

StatusForm->EventListBox->Items->Add("Flags: "+IntToStr((int)Flags));
StatusForm->EventListBox->Items->Add("FromPlayer: "+IntToStr((int)FromPlayer));
StatusForm->EventListBox->Items->Add("ToPlayer: "+IntToStr((int)ToPlayer));
StatusForm->EventListBox->Items->Add("ToGroup: "+IntToStr((int)ToGroup));
AnsiString temp = Chat->MessageA;
StatusForm->EventListBox->Items->Add("Message: "+temp );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1CreatePlayerOrGroup(TObject *Sender,
	DWORD PlayerType, DWORD Id, DWORD CurrentPlayers, void *Data, DWORD DataSize,
	TDPName *Name, DWORD IdParent, DWORD Flags)
{
StatusForm->EventListBox->Items->Add("OnCreatePlayerOrGroup");
if (PlayerType==DPPLAYERTYPE_PLAYER)
	{
//	PlayerNamesListBox->Items->Add( "["+IntToStr((int) Id)+"] "+Name->ShortNameA +", "+ Name->LongNameA );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1DeleteGroupFromGroup(TObject *Sender,
	DWORD IdParentGroup, DWORD IdGroup)
{
StatusForm->EventListBox->Items->Add("OnDeleteGroupFromGroup");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1DeletePlayerFromGroup(TObject *Sender,
	DWORD IdParentGroup, DWORD IdGroup)
{
StatusForm->EventListBox->Items->Add("OnDeletePlayerFromGroup");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1DestroyPlayerOrGroup(TObject *Sender,
	DWORD PlayerType, DWORD Id, void *LocalData, DWORD LocalDataSize,
	void *RemoteData, DWORD RemoteDataSize, TDPName *Name, DWORD IdParent,
	DWORD dwFlags)
{
StatusForm->EventListBox->Items->Add("OnDestroyPlayerOrGroup");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumGroupPlayers(TObject *Sender, DWORD Id,
	DWORD PlayerType, TDPName *Name, DWORD Flags, void *Context, bool &Continue)
{
if (NumPlayersInGroups<MAX_PLAYERS_IN_GROUPS)
	{
    PlayerInGroupsIDs[NumPlayersInGroups] = Id;
    PlayerInGroupsPlayerTypes[NumPlayersInGroups] = PlayerType;
    PlayerInGroupsNames[NumPlayersInGroups] = new TDPName(this);
    if (PlayerInGroupsNames[NumPlayersInGroups])
    	{
        PlayerInGroupsNames[NumPlayersInGroups]->ShortNameA = Name->ShortNameA;
        PlayerInGroupsNames[NumPlayersInGroups]->LongNameA = Name->LongNameA;
        }
    PlayerInGroupsFlags[NumPlayersInGroups] = Flags;
    NumPlayersInGroups++;
    }
else
	Continue = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumGroups(TObject *Sender, DWORD Id,
	DWORD PlayerType, TDPName *Name, DWORD Flags, void *Context, bool &Continue)
{
if (NumGroups<MAX_GROUPS)
	{
    GroupIDs[NumGroups] = Id;
    GroupPlayerTypes[NumGroups] = PlayerType;
    GroupNames[NumGroups] = new TDPName(this);
    if (GroupNames[NumGroups])
    	{
        GroupNames[NumGroups]->ShortNameA = Name->ShortNameA;
        GroupNames[NumGroups]->LongNameA = Name->LongNameA;
        }
    GroupFlags[NumGroups] = Flags;
    PlayerSetupForm->EnumGroupsListBox->Items->Add( IntToStr( Id ) + ", " + Name->ShortNameA );
    NumGroups++;
    }
else
	Continue = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumGroupsInGroup(TObject *Sender, DWORD Id,
	DWORD PlayerType, TDPName *Name, DWORD Flags, void *Context, bool &Continue)
{
if (NumGroupsInGroups<MAX_GROUPS_IN_GROUPS)
	{
    GroupsInGroupsIDs[NumGroupsInGroups] = Id;
    GroupsInGroupsPlayerTypes[NumGroupsInGroups] = PlayerType;
    GroupsInGroupsNames[NumGroupsInGroups] = new TDPName(this);
    if (GroupsInGroupsNames[NumGroupsInGroups])
    	{
        GroupsInGroupsNames[NumGroupsInGroups]->ShortNameA = Name->ShortNameA;
        GroupsInGroupsNames[NumGroupsInGroups]->LongNameA = Name->LongNameA;
        }
    GroupsInGroupsFlags[NumGroupsInGroups] = Flags;
    PlayerSetupForm->EnumGroupsInGroupsListBox->Items->Add( IntToStr( Id ) + ", " + Name->ShortNameA );
    NumGroupsInGroups++;
    }
else
	Continue = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1EnumPlayers(TObject *Sender, DWORD Id,
	DWORD PlayerType, TDPName *Name, DWORD Flags, void *Context, bool &Continue)
{
if (NumPlayers<MAX_PLAYERS)
	{
    PlayerIDs[NumPlayers] = Id;
    PlayerPlayerTypes[NumPlayers] = PlayerType;
    PlayerNames[NumPlayers] = new TDPName(this);
    if (PlayerNames[NumPlayers])
    	{
        PlayerNames[NumPlayers]->ShortNameA = Name->ShortNameA;
        PlayerNames[NumPlayers]->LongNameA = Name->LongNameA;
        }
    PlayerFlags[NumPlayers] = Flags;
    NumPlayers++;
    }
else
	Continue = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1Host(TObject *Sender)
{
StatusForm->EventListBox->Items->Add("OnHost");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1SecureMessage(TObject *Sender, DWORD Flags,
	DWORD IdFrom, void *Data, DWORD DataSize)
{
StatusForm->EventListBox->Items->Add("OnSecureMessage");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1SendComplete(TObject *Sender, DWORD From,
	DWORD To, DWORD Flags, DWORD Priority, DWORD Timeout, void *Context,
	DWORD MsgID, HRESULT hr, DWORD SendTime)
{
StatusForm->EventListBox->Items->Add("OnSendComplete");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1SetGroupOwner(TObject *Sender, DWORD Group,
	DWORD NewOwner, DWORD OldOwner)
{
StatusForm->EventListBox->Items->Add("OnSetGroupOwner");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1SetPlayerOrGroupData(TObject *Sender,
	DWORD Flags, DWORD IdFrom, void *Data, DWORD DataSize)
{
StatusForm->EventListBox->Items->Add("OnSetPlayerOrGroupData");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1SetPlayerOrGroupName(TObject *Sender,
	DWORD PlayerType, DWORD Id, TDPName *Name)
{
StatusForm->EventListBox->Items->Add("OnSetPlayerOrGroupName");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Play1SetSessionDesc(TObject *Sender,
	TDPSessionDesc *Desc)
{
StatusForm->EventListBox->Items->Add("OnSetSessionDesc");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ConnectionsListBoxClick(TObject *Sender)
{
SelectedConnectionLabel->Caption = ConnectionNames[ ConnectionsListBox->ItemIndex ]->ShortNameA;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BitBtn1Click(TObject *Sender)
{
if (ConnectionInitialized)
	{
    FormDestroy(this);
    FormCreate(this);
    ConnectionInitialized = false;
    }

ConnectionInitialized = Dx_Play1->InitializeConnection( ConnectionBuffers[ ConnectionsListBox->ItemIndex ] );
StatusForm->ConnectionInitializedLabel->Caption = (ConnectionInitialized) ? "Initialized" : "(not initialized)";

if (ConnectionInitialized)
	{
    CapsListBox->Items->Clear();
    if (Dx_Play1->GetCaps( DPCaps1, 0 ))
        {
        CapsListBox->Items->Add( "Flags= ");
        CapsListBox->Items->AddStrings( DPCaps1->Flags_Strings );
        CapsListBox->Items->Add( "HeaderLength= "+IntToStr( (int) DPCaps1->HeaderLength ) );
        CapsListBox->Items->Add( "HundredBaud= "+IntToStr( (int) DPCaps1->HundredBaud ) );
        CapsListBox->Items->Add( "Latency= "+IntToStr( (int) DPCaps1->Latency ) );
        CapsListBox->Items->Add( "MaxBufferSize= "+IntToStr( (int) DPCaps1->MaxBufferSize ) );
        CapsListBox->Items->Add( "MaxLocalPlayer= "+IntToStr( (int) DPCaps1->MaxLocalPlayers ) );
        CapsListBox->Items->Add( "MaxPlayers= "+IntToStr( (int) DPCaps1->MaxPlayers ) );
        CapsListBox->Items->Add( "Tag= "+IntToStr( (int) DPCaps1->Tag ) );
        }
    else
        CapsListBox->Items->Add( "ERROR: Could not retrieve caps" );

    sleep(1);
	SessionsListBox->Items->Clear();
	NumSessions = 0;
	DPSessionDesc1->Clear();
	Dx_Play1->EnumSessions( DPSessionDesc1, 0, NULL, DPENUMSESSIONS_ALL | DPENUMSESSIONS_ASYNC );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BitBtn2Click(TObject *Sender)
{
if (ConnectionInitialized)
	{
    FormDestroy(this);
    FormCreate(this);
    ConnectionInitialized = false;
    CapsListBox->Items->Clear();
	SelectedConnectionLabel->Caption = "(none selected)";
    }

StatusForm->ConnectionInitializedLabel->Caption = (ConnectionInitialized) ? "Initialized" : "(not initialized)";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
// cleanup
if (Dx_Play1->Created) Dx_Play1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
// Initialize DirectPlay interface
Dx_Play1->UseAnsiString = true;
Dx_Play1->Create();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BitBtn6Click(TObject *Sender)
{
for (int i=0;i<NumSessions;i++) if (Sessions[i]) delete Sessions[i];

SessionsListBox->Items->Clear();
JoinSessionButton->Enabled = false;
SelectedSessionLabel->Caption = "(none selected)";

NumSessions = 0;
DPSessionDesc1->Clear();
Dx_Play1->EnumSessions( DPSessionDesc1, 0, NULL, DPENUMSESSIONS_ALL | DPENUMSESSIONS_ASYNC );

StatusForm->SessionStatusLabel->Caption = IntToStr(NumSessions)+" Sessions";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MaxPlayersEditChange(TObject *Sender)
{
try {
	int i = StrToInt( MaxPlayersEdit->Text );
    }
catch (...)
	{
    MaxPlayersEdit->Text = "10";
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SessionsListBoxClick(TObject *Sender)
{
SelectedSessionLabel->Caption = SessionsListBox->Items->Strings[SessionsListBox->ItemIndex];
JoinSessionButton->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
Top=StatusForm->Height+1;
Left=0;
}
//---------------------------------------------------------------------------
